@@ -3,22 +3,19 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
+ * The contents of this file constitute Original Code as defined in and
+ * are subject to the Apple Public Source License Version 1.1 (the
+ * "License").  You may not use this file except in compliance with the
+ * License.  Please obtain a copy of the License at
+ * http://www.apple.com/publicsource and read it before using this file.
  * 
- * This file contains Original Code and/or Modifications of Original Code
- * as defined in and that are subject to the Apple Public Source License
- * Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
- * 
- * The Original Code and all software distributed under the License are
- * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This Original Code and all software distributed under the License are
+ * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
- * Please see the License for the specific language governing rights and
- * limitations under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
+ * License for the specific language governing rights and limitations
+ * under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -175,6 +172,13 @@
 #define MII_BCM5221_TestRegister		0x1F
 #define MII_BCM5221_ShadowRegEnableBit 		0x0080
 
+	/* MII BCM5241 ID:			*/
+
+#define MII_BCM5241_ID		0x0143BC30
+#define MII_BCM5241_MASK	0xFFFFFFF0
+
+#define MII_BCM5241_StandbyPowerMode	0x0008	// in MII_BCM5221_AuxiliaryMode4
+
 
 	/*** MII LXT971 (Level One) Specific:	***/
 
@@ -289,5 +293,5 @@
 
 //#define MII_LINK_TIMEOUT	2500
 ///#define MII_LINK_TIMEOUT	5000	/// latest Marvell needs more than 2.5 secs
-#define MII_LINK_TIMEOUT	10000	/// Broadcom 5421 needs more than 6.4 secs
+#define MII_LINK_TIMEOUT	7000	/// Broadcom 5421 needs more than 6.4 secs
 #define MII_LINK_DELAY		50
